import json
import os
from difflib import SequenceMatcher
from unicodedata import normalize
from collections import OrderedDict
import platform

_system = platform.system()


def get_application_dir(appname='coupon'):
    if _system == 'Linux':
        appdir = os.path.join(os.environ['HOME'], '.' + appname)
    elif _system == 'Windows':
        appdir = os.path.join(os.environ['APPDATA'], appname)
    if not os.path.exists(appdir):
        os.mkdir(appdir)
    return appdir


def remove_accentuation(string):
    """Remove the accentuantion of a string.
    @returns: the string without accentuantion.
    """
    # Taken from http://www.python.org.br/wiki/RemovedorDeAcentos
    return normalize('NFKD', string).encode('ASCII', 'ignore').decode("utf-8")


def store_key_name(name):
    return remove_accentuation(name).upper().replace(' ', '')


def similar(a, b):
    """
    Uses the Ratcliff/Obershelp algorithm to compute the doubled number of
    matching characters divided by the total number of characters in the two
    strings or sequences.

    Sources:
        - https://stackoverflow.com/questions/6690739/fuzzy-string-comparison-in-python-confused-with-which-library-to-use
        - https://stackoverflow.com/questions/17388213/find-the-similarity-percent-between-two-strings

    :param a: string or sequence to compare.
    :param b: string or sequence to compare.
    :return: a ratio between 0 and 1 of the match.
    """
    return SequenceMatcher(None, a, b).ratio()


class BaseScrapingClass(object):
    @classmethod
    def get_store_list(cls):
        """
        Returns a dict store_name:url
        """
        raise NotImplementedError

    @classmethod
    def get_url_for_store(cls, store_name):
        raise NotImplementedError

    @classmethod
    def get_url_for_category(cls, category_name):
        raise NotImplementedError

    # @classmethod
    # def get_afiliate_url(cls, store_name):
    #     store_name = remove_accentuation(store_name).upper().replace(' ', '')
    #     store_dict = all_stores.get(store_name)
    #     if not store_dict:
    #         return '...'
    #     return all_stores.get(store_name).get(URL_SUPER_CUPOM_BOT)

    def _is_valid_coupon(self, coupon):
        # filter invalid coupons
        # as vezes o site gera um cupom proprio, que so vale pelo link deles
        raise NotImplementedError

    @property
    def store_coupons(self):
        raise NotImplementedError

    def _fix_description(self, description):
        return description.replace('Aniversário Groupon → ', '')

    def _fix_expire_date(self, expire_date):
        return expire_date.replace('Expira em N/A', 'Indeterminado')

    def _fix_store_name(self, store_name):
        return store_name \
            .replace('.com', '') \
            .replace('.br', '') \
            .replace('Cupom de descontos ', '') \
            .replace('Cupom ', '') \
            .replace(';', '') \
            .replace('Livraria ', '') \
            .replace('</em>', '') \
            .replace('Lojas ', '') \
            .replace(' Brasil', '') \
            .replace('Sobre ', '')


basedir = os.path.dirname(__file__)


def get_all_stores():
    # TODO create some config file to setup some paths
    try:
        with open(os.path.join(basedir, '../resources/all_stores.json'), 'r') as f:
            d = json.load(f, object_pairs_hook=OrderedDict)
    except FileNotFoundError:
        print('File does not exist, please create {}'.format(os.path.join(basedir, '../resources/all_stores.json')))
    except IOError:
        print('File is not accessible')
    return d


all_stores = get_all_stores()
